# -​*- coding: utf-8 -*​-

import requests

PAYBOOK_URL = 'https://sync.paybook.com/v1/'

class Paybook():

	api_key = None
	logger = None
	print_calls = False
	__INDENT__ = '   '

	def __init__(self,api_key=None,logger=None,print_calls=False):
		Paybook.api_key = api_key
		Paybook.logger = logger
		Paybook.print_calls = print_calls#For debugging

	@staticmethod
	def __call__(endpoint=None,method=None,data=None,params=None,headers=None,url=None):
		Paybook.log('API Call: ',call=True)
		if url is None:
			url = PAYBOOK_URL + endpoint
		Paybook.log(Paybook.__INDENT__ + 'API Key:     ' + str(Paybook.api_key),call=True)
		Paybook.log(Paybook.__INDENT__ + 'Endpoint:    ' + str(url),call=True)
		Paybook.log(Paybook.__INDENT__ + 'HTTP Method: ' + str(method),call=True)
		Paybook.log(Paybook.__INDENT__ + 'Data:        ' + str(data),call=True)
		Paybook.log(Paybook.__INDENT__ + 'Params:      ' + str(params),call=True)
		Paybook.log(Paybook.__INDENT__ + 'Headers:     ' + str(headers),call=True)
		if method == 'post':
			conn = requests.post(url,data=data,params=params,headers=headers)
		elif method == 'get':
			conn = requests.get(url,data=data,params=params,headers=headers)
		elif method == 'delete':
			conn = requests.delete(url,data=data,params=params,headers=headers)
		Paybook.log(Paybook.__INDENT__ + 'API Response code: ' + str(conn.status_code),call=True)
		if conn.status_code == 200:
			Paybook.log('Success response: ')
			try:# JSON responses:
				Paybook.log(Paybook.__INDENT__ + str(conn.json()))
				return conn.json()['response']
			except Exception as e:# Attachments:
				Paybook.log(Paybook.__INDENT__ + str(conn.content))
				return conn.content
		else:
			Paybook.log('Error catched')
			raise Paybook.__get_api_error__(conn)

	@staticmethod
	def __get_api_error__(conn):
		try:
			api_error = Error(conn.status_code,'','','')
		except Exception as e:
			api_error = Error(500,'Connection Error')
		return api_error

	@staticmethod
	def log(message,call=False):
		if Paybook.print_calls == True and call == True:
			print message
		if Paybook.logger is not None:
			Paybook.logger.debug(message)

class Error(Exception):

	def __init__(self,code,response,message,status):
		self.code = code
		self.response = response
		self.message = message
		self.status = status

	def get_json(self):
		return {
			'code' : self.code,
			'response' : self.response,
			'message' : self.message,
			'status' : self.status
		}# End of return







# -​*- coding: utf-8 -*​-

from json import dumps
import main

class Credentials(main.Paybook):

	def __init__(self,session=None,id_user=None,id_site=None,credentials=None,credentials_json=None):# If it already exists it retrieves the existing one
		Credentials.log('\n')
		Credentials.log('Credentials.__init__')
		self.id_site = None
		self.twofa_config = None
		if credentials_json is None:
			if id_user is not None:
				data = {
					'api_key' : Credentials.api_key,
					'id_user' : id_user
				}#End of data
			else:
				data = {
					'token' : session.token
				}#End of data
			data['id_site'] = id_site
			data['credentials'] = credentials
			credentials_json = Credentials.__call__(endpoint='credentials',method='post',data=dumps(data))
		if id_site is not None:
			self.id_site = id_site
		elif 'id_site' in credentials_json:
			self.id_site = credentials_json['id_site']
		self.id_credential = credentials_json['id_credential']
		self.username = credentials_json['username']
		self.id_site_organization = credentials_json['id_site_organization'] if 'id_site_organization' in credentials_json else None
		self.id_site_organization_type = credentials_json['id_site_organization_type'] if 'id_site_organization_type' in credentials_json else None
		self.ws = credentials_json['ws'] if 'ws' in credentials_json else None
		self.status = credentials_json['status'] if 'status' in credentials_json else None
		self.twofa = credentials_json['twofa'] if 'twofa' in credentials_json else None

	@staticmethod
	def delete(session=None,id_user=None,id_credential=None):
		Credentials.log('\n')
		Credentials.log('Credentials.delete')
		if id_user is not None:
			params = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		delete_response = Credentials.__call__(endpoint='credentials/'+id_credential,method='delete',params=params)
		return True

	@staticmethod
	def get(session=None,id_user=None):
		Credentials.log('\n')
		Credentials.log('Credentials.get')
		if id_user is not None:
			params = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		credentials_jsons = Credentials.__call__(endpoint='credentials',method='get',params=params)
		credentials_list = []
		for credentials_json in credentials_jsons:
			credentials = Credentials(credentials_json=credentials_json)
			credentials_list.append(credentials)
		return credentials_list

	def get_status(self,session=None,id_user=None):
		Credentials.log('\n')
		Credentials.log('Credentials.status')
		headers = {'Content-type' : 'application/json'}		
		if id_user is not None:
			data = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of data
		else:
			data = {
				'token' : session.token
			}#End of data
		if self.id_site is not None:
			data['id_site'] = self.id_site
			status = Credentials.__call__(method='get',data=dumps(data),headers=headers,url=self.status)
			for each_status in status:
				code = each_status['code']
				if code == 410:
					self.twofa = each_status['address']
					self.twofa_config = each_status['twofa'][0]
			return status

	def set_twofa(self,session=None,id_user=None,twofa_value=None):
		Credentials.log('\n')
		Credentials.log('Credentials.twofa')
		headers = {'Content-type' : 'application/json'}		
		if id_user is not None:
			data = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of data
		else:
			data = {
				'token' : session.token
			}#End of data
		if self.id_site:
			data['id_site'] = self.id_site
			data['twofa'] = {}
			data['twofa'][self.twofa_config['name']] = twofa_value
			Credentials.__call__(method='post',data=dumps(data),url=self.twofa)
			return True

	def get_json(self):
		return {
			'id_site' : self.id_site,
			'id_site_organization' : self.id_site_organization,
			'id_site_organization_type' : self.id_site_organization_type,
			'id_credential' : self.id_credential,
			'status' : self.status,
			'twofa' : self.twofa,
			'ws' : self.ws,
			'username' : self.username
		}#End of return

	# @staticmethod# Pending because it is not implemented in REST API yet
	# def status(session=None,id_user=None,id_credential=None):
	# 	Credentials.log('\n')
	# 	Credentials.log('Credentials.get_status')
	# 	if id_user is not None:
	# 		params = {
	# 			'api_key' : Credentials.api_key,
	# 			'id_user' : id_user
	# 		}#End of params
	# 	else:
	# 		params = {
	# 			'token' : session.token
	# 		}#End of params
	# 	status_json = Credentials.__call__(endpoint='credentials/'+id_credential+'/status',method='get',params=params)
	# 	print status_json











# -​*- coding: utf-8 -*​-

import main

class Session(main.Paybook):

	def __init__(self,user=None,token=None):
		self.user = user
		if token is not None:
			self.token = token
			self.iv = None
			self.key = None
		else:
			Session.log('\n')
			Session.log('Session.__init__')
			data = {
				'api_key' : Session.api_key, 
				'id_user' : self.user.id_user
			}#End of data
			session_json = Session.__call__(endpoint='sessions',method='post',data=data)
			self.token = session_json['token'] if 'token' in session_json else None
			self.iv = session_json['iv'] if 'iv' in session_json else None
			self.key = session_json['key'] if 'key' in session_json else None

	def verify(self):
		Session.log('\n')
		Session.log('Session.verify')
		Session.__call__(endpoint='sessions/' + self.token + '/verify',method='get')# If not, raise Error
		return True

	@staticmethod
	def delete(token=None):
		Session.log('\n')
		Session.log('Session.delete')
		Session.__call__(endpoint='sessions/' + token + '',method='delete')# If error, raise Error
		return True

	def set_token(self,token):
		self.token = token

	def get_json(self):
		return {
			'iv' : self.iv,
			'key' : self.key,
			'token' : self.token
		}#End of return




# -​*- coding: utf-8 -*​-

import main

class Account(main.Paybook):

	def __init__(self,account_json):
		self.id_account = account_json['id_account']
		self.id_user = account_json['id_user']
		self.id_external = account_json['id_external']
		self.id_credential = account_json['id_credential']
		self.id_site = account_json['id_site']
		self.id_site_organization = account_json['id_site_organization']
		self.name = account_json['name']
		self.number = account_json['number']
		self.balance = account_json['balance']
		self.site = account_json['site']
		self.dt_refresh = account_json['dt_refresh']
		self.extra = account_json['extra'] if 'extra' in account_json else {}

	@staticmethod
	def get(session=None,id_user=None,options={}):
		if options is True:
			return {
				'id_account' : 'str',
				'id_credential' : 'str',
				'id_site' : 'str',
				'id_site_organization' : 'str',
				'id_site_organization_type' : 'str',
				'id_site_organization' : 'str',
				'fields' : 'str',
				'limit' : 'int',
				'skip' : 'int',
				'order' : 'order'
			}#End of return
		Account.log('\n')
		Account.log('Account.get')
		params = options
		if id_user is not None:
			params['api_key'] = Credentials.api_key
			params['id_user'] = id_user
		else:
			params['token'] = session.token
		account_jsons = Account.__call__(endpoint='accounts',method='get',params=params)
		accounts = []
		for account_json in account_jsons:
			account = Account(account_json)
			accounts.append(account)
		return accounts

	def get_json(self):
		return {
			'extra' : self.extra,
			'id_account' : self.id_account,
			'id_user' : self.id_user,
			'id_external' : self.id_external,
			'id_credential' : self.id_credential,
			'id_site' : self.id_site,
			'id_site_organization' : self.id_site_organization,
			'name' : self.name,
			'number' : self.number,
			'balance' : self.balance,
			'site' : self.site,
			'dt_refresh' : self.dt_refresh
		}#End of return		





# -​*- coding: utf-8 -*​-

from paybook import main
from paybook import user
from paybook import session
from paybook import account
from paybook import attachment
from paybook import catalogues
from paybook import credentials
from paybook import transaction

# Core classes:
Error = main.Error
Paybook = main.Paybook
User = user.User
Session = session.Session

# Data classes
Account = account.Account
Attachment = attachment.Attachment
Catalogues = catalogues.Catalogues
Credentials = credentials.Credentials
Transaction = transaction.Transaction



# -​*- coding: utf-8 -*​-

import main

class User(main.Paybook):

	def __init__(self,name=None,id_user=None,id_external=None,user_json=None):# If it already exists it retrieves the existing one
		User.log('\n')
		User.log('User.__init__')
		if user_json is None:
			if id_user is None and name is not None:# New user
				User.log('Creating new user ... ')
				data = {
					'api_key' : User.api_key, 
					'name' : name
				}#End of data
				if id_external is not None:
					data['id_external'] = id_external
				user_json = User.__call__(endpoint='users',method='post',data=data)
			elif id_user is not None:# Existing user
				User.log('Retrieveing existing user ... ')
				existing_users = User.get()
				for existing_user in existing_users:
					id_existing_user = existing_user.id_user
					if id_existing_user == id_user:
						user_json = existing_user.get_json()
		self.name = user_json['name']
		self.id_user = user_json['id_user']
		self.id_external = user_json['id_external']
		self.dt_create = user_json['dt_create']
		self.dt_modify = user_json['dt_modify']

	@staticmethod
	def delete(id_user=None):
		User.log('\n')
		User.log('User.delete')
		params = {
			'api_key' : User.api_key
		}#End of params
		delete_response = User.__call__(endpoint='users/'+id_user,method='delete',params=params)
		return True

	@staticmethod
	def get(options=None):
		User.log('\n')
		User.log('User.get')
		params = {
			'api_key' : User.api_key
		}#End of params
		user_jsons = User.__call__(endpoint='users',method='get',params=params)
		users = []
		for user_json in user_jsons:
			user = User(user_json=user_json)
			users.append(user)
		return users

	def get_json(self):
		return {
			'name' : self.name,
			'id_user' : self.id_user,
			'id_external' : self.id_external,
			'dt_create' : self.dt_create,
			'dt_modify' : self.dt_modify
		}#End of return



# -​*- coding: utf-8 -*​-

import main

class Attachment(main.Paybook):

	def __init__(self,attachment_json):
		self.id_account = attachment_json['id_account']
		self.id_user = attachment_json['id_user']
		self.id_external = attachment_json['id_external']
		self.id_attachment_type = attachment_json['id_attachment_type']
		self.id_transaction = attachment_json['id_transaction']
		self.file = attachment_json['file']
		self.extra = attachment_json['extra'] if 'extra' in attachment_json else None
		self.url = attachment_json['url']
		self.dt_refresh = attachment_json['dt_refresh']

	@staticmethod
	def get(session=None,id_user=None,options=None,id_attachment=None,extra=None):
		Attachment.log('\n')
		Attachment.log('Attachment.get')
		if id_user is not None:
			params = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		if id_attachment:# Attachment content
			if extra is True:# Extracted data (not the content)
				id_attachment = id_attachment + '/extra'
			attachment_data = Attachment.__call__(endpoint=id_attachment,method='get',params=params)
			return attachment_data
		else:
			attachment_jsons = Attachment.__call__(endpoint='attachments',method='get',params=params)
		attachments = []
		for attachment_json in attachment_jsons:
			attachment = Attachment(attachment_json)
			attachments.append(attachment)
		return attachments

	@staticmethod
	def get_count(session=None,id_user=None,options=None):
		Attachment.log('\n')
		Attachment.log('Attachment.get_count')
		if id_user is not None:
			params = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params			
		attachments_count_json = Attachment.__call__(endpoint='attachments/count',method='get',params=params)
		attachments_count = attachments_count_json['count']
		return attachments_count





# -​*- coding: utf-8 -*​-

import main

class Catalogues(main.Paybook):

	@staticmethod
	def get_account_types(session=None,id_user=None,options=None):
		Catalogues.log('\n')
		Catalogues.log('Catalogues.get_account_types')
		if id_user is not None:
			params = {
				'api_key' : Catalogues.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		account_type_jsons = Catalogues.__call__(endpoint='catalogues/account_types',method='get',params=params)
		account_types = []
		for account_type_json in account_type_jsons:
			account_type = Account_type(account_type_json)
			account_types.append(account_type)
		return account_types

	@staticmethod
	def get_attachment_types(session=None,id_user=None,options=None):
		Catalogues.log('\n')
		Catalogues.log('Catalogues.get_attachment_types')
		if id_user is not None:
			params = {
				'api_key' : Catalogues.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		attachment_type_jsons = Catalogues.__call__(endpoint='catalogues/attachment_types',method='get',params=params)
		attachment_types = []
		for attachment_type_json in attachment_type_jsons:
			attachment_type = Attachment_type(attachment_type_json)
			attachment_types.append(attachment_type)
		return attachment_types

	@staticmethod
	def get_countries(session=None,id_user=None,options=None):
		Catalogues.log('\n')
		Catalogues.log('Catalogues.get_countries')
		if id_user is not None:
			params = {
				'api_key' : Catalogues.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		country_jsons = Catalogues.__call__(endpoint='catalogues/countries',method='get',params=params)
		countries = []
		for country_json in country_jsons:
			country = Country(country_json)
			countries.append(country)
		return countries

	@staticmethod
	def get_sites(session=None,id_user=None,options=None,is_test=False):
		Catalogues.log('\n')
		Catalogues.log('Catalogues.get_sites')
		if id_user is not None:
			params = {
				'api_key' : Catalogues.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		if is_test is True:
			params['is_test'] = is_test
		site_jsons = Catalogues.__call__(endpoint='catalogues/sites',method='get',params=params)
		sites = []
		for site_json in site_jsons:
			site = Site(site_json)
			sites.append(site)
		return sites
 
	@staticmethod
	def get_site_organizations(session=None,id_user=None,options=None,is_test=False):
		Catalogues.log('\n')
		Catalogues.log('Catalogues.get_site_organizations')
		if id_user is not None:
			params = {
				'api_key' : Catalogues.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		if is_test is True:
			params['is_test'] = is_test
		site_organization_jsons = Catalogues.__call__(endpoint='catalogues/site_organizations',method='get',params=params)
		site_organizations = []
		for site_organization_json in site_organization_jsons:
			site_organization = Site_organization(site_organization_json)
			site_organizations.append(site_organization)
		return site_organizations

class Account_type():

	def __init__(self,account_type_json):
		self.id_account_type = account_type_json['id_account_type']
		self.name = account_type_json['name']

class Attachment_type():

	def __init__(self,attachment_type_json):
		self.id_attachment_type = attachment_type_json['id_attachment_type']
		self.name = attachment_type_json['name']

class Country():

	def __init__(self,country_json):
		self.id_country = country_json['id_country']
		self.name = country_json['name']
		self.code = country_json['code']

class Site():

	def __init__(self,site_json):
		self.id_site = site_json['id_site']
		self.id_site_organization = site_json['id_site_organization']
		self.id_site_organization_type = site_json['id_site_organization_type']
		self.name = site_json['name']
		credentials_structures = []
		for credential_structure_json in site_json['credentials']:
			credentials_structure = Credentials_structure(credential_structure_json)
			credentials_structures.append(credentials_structure)
		self.credentials = credentials_structures
		self.site_json = site_json

	def get_json(self):
		return self.site_json

class Credentials_structure():

	def __init__(self,credentials_structure_json):
		self.name = credentials_structure_json['name']
		self.type = credentials_structure_json['type']
		self.label = credentials_structure_json['label']
		self.required = credentials_structure_json['required']
		self.username = credentials_structure_json['username']
		self.validation = None

class Site_organization():

	def __init__(self,site_organization_json):
		self.id_site_organization = site_organization_json['id_site_organization']
		self.id_site_organization_type = site_organization_json['id_site_organization_type']
		self.id_country = site_organization_json['id_country']
		self.name = site_organization_json['name']
		self.avatar = site_organization_json['avatar']
		self.small_cover = site_organization_json['small_cover']
		self.cover = site_organization_json['cover']
		self.site_organization_json = site_organization_json

	def get_json(self):
		return self.site_organization_json

































# -​*- coding: utf-8 -*​-

import main

class Transaction(main.Paybook):

	def __init__(self,transaction_json):
		self.id_transaction = transaction_json['id_transaction']
		self.id_user = transaction_json['id_user']
		self.id_site = transaction_json['id_site']
		self.id_site_organization = transaction_json['id_site_organization']
		self.id_site_organization_type = transaction_json['id_site_organization_type']
		self.id_account = transaction_json['id_account']
		self.id_account_type = transaction_json['id_account_type']
		self.id_currency = transaction_json['id_currency']
		self.is_disable = transaction_json['is_disable']
		self.description = transaction_json['description']
		self.amount = transaction_json['amount']
		self.currency = transaction_json['currency']
		self.attachments = transaction_json['attachments']
		self.extra = transaction_json['extra']
		self.reference = transaction_json['reference']
		self.dt_transaction = transaction_json['dt_transaction']
		self.dt_refresh = transaction_json['dt_refresh']

	@staticmethod
	def get(session=None,id_user=None,options={}):
		if options is True:
			return {
				'id_transaction' : 'str',
				'id_account' : 'str',
				'id_credential' : 'str',
				'id_site' : 'str',
				'id_site_organization' : 'str',
				'id_site_organization_type' : 'str',
				'is_disable' : 'int',
				'dt_refresh_from' : 'str',
				'dt_refresh_to' : 'str',
				'dt_transaction_from' : 'str',
				'dt_transaction_to' : 'str',
				'fields' : 'str',
				'limit' : 'int',
				'skip' : 'int',
				'order' : 'int'
			}#End of return
		Transaction.log('\n')
		Transaction.log('Transaction.get')
		params = options
		if id_user is not None:
			params['api_key'] = Transaction.api_key
			params['id_user'] = id_user
		else:
			params['token'] = session.token
		transaction_jsons = Transaction.__call__(endpoint='transactions',method='get',params=params)
		transactions = []
		for transaction_json in transaction_jsons:
			transaction = Transaction(transaction_json)
			transactions.append(transaction)
		return transactions

	@staticmethod
	def get_count(session=None,id_user=None,options=None):
		Transaction.log('\n')
		Transaction.log('Transaction.get_count')
		if id_user is not None:
			params = {
				'api_key' : Credentials.api_key,
				'id_user' : id_user
			}#End of params
		else:
			params = {
				'token' : session.token
			}#End of params
		count_json = Transaction.__call__(endpoint='transactions/count',method='get',params=params)
		transactions_count = count_json['count']
		return transactions_count
	
	def get_json(self):
		return {
			'id_transaction' : self.id_transaction,
			'id_user' : self.id_user,
			'id_site' : self.id_site,
			'id_site_organization' : self.id_site_organization,
			'id_site_organization_type' : self.id_site_organization_type,
			'id_account' : self.id_account,
			'id_account_type' : self.id_account_type,
			'id_currency' : self.id_currency,
			'is_disable' : self.is_disable,
			'description' : self.description,
			'amount' : self.amount,
			'currency' : self.currency,
			'attachments' : self.attachments,
			'extra' : self.extra,
			'reference' : self.reference,
			'dt_transaction' : self.dt_transaction,
			'dt_refresh' : self.dt_refresh,
		}#End of return




		
